==> broker-admin-0.5-sources.jar: webapp/requests.js <==

function init(){
	$('#new_request').on('submit', function(e){
		e.preventDefault();
		addNewRequest();
	});
	loadRequestList();
	$('#new_request input[name="scheduled"]').val(new Date().toDateInputValue());
	$('#new_request input[name="p_name"]').val('Ihr Name')
	$('#new_request input[name="p_email"]').val('ihre.email@addres.se');
	$('#new_request input[name="title"]').val('Titel der Abfrage')
	$('#new_request input[name="x_ts"]').val(new Date().toDateInputValue()+"T00:00");
	// load nodes
	getNodes(function(nodes){
		for( var i=0; i<nodes.length; i++ ){
			$('#target').append($("<option/>",{
	        	value: nodes[i].id,
	        	text: nodes[i].cn,
	        	title: nodes[i].dn
	        }));
		}
		console.log('Loaded nodes: '+nodes.length);
	},function(){
		alert('DWH-Knoten konnten nicht geladen werden.');
	});
	$('#limit_target_a').click(function(){
		// clear selection
		$("#target option:selected").prop("selected", false);
		$("#target").prop("disabled", true);
	});
	$('#limit_target_a').prop("checked", true).trigger("click");
	$('#limit_target_s').click(function(){
		// clear selection
		$("#target").prop("disabled", false);
	});
	// hide new request form
	$('#create').before($('<button/>',{text:'Show form'}).click(function(){
		$(this).remove();
		$('#create').show();
	}));
	$('#create').hide();
	// add button to set date
	$('#new_request input[name="reference"]').after($('<a>Last month</a>').click(function(){
		var d = new Date();
		d.setDate(1);
		console.log('Setting reference date',d);
		$('#new_request input[name="reference"]').val(d.toDateInputValue()+"T00:00:00");
	}));
	// switch between single/repeating executions
	$('#x_exec legend:first-child').css('cursor','pointer').click(function(){
		var rep = ($('#x_qid').parent('fieldset').length == 1);
		var target;
		if( rep ){
			// switch to single
			target = $('#hidden_fields');
			$(this).text('Single execution (click to change)');
		}else{
			// switch to repeating
			target = $('#x_exec');
			$(this).text('Repeated execution (click to change)');
		}
		$('#x_intvl').detach().appendTo(target);
		$('#x_qid').detach().appendTo(target);
	});
	
}

//$(document).ready(function(){
//	init();
//});
Date.prototype.toDateInputValue = (function(){
	var local = new Date(this);
	local.setMinutes(this.getMinutes() - this.getTimezoneOffset());
	return local.toJSON().slice(0,10);
});
function loadRequestList(){
	$.get({
		url: rest_base+'/broker/request',
		success: function(data) {
			// clear list
			$('#requests').empty();
			// TODO render request list
			xml = $(data);
			xml.find('request').each(function(){
				var id = $(this).attr('id');
				var cls = 'req';
				if( $(this).find('published').text() != '' ){
					cls += ' published';
				}
				if( $(this).find('closed').text() != '' ){
					cls += ' closed';
				}
				var el = $('<div class="'+cls+'"><span>request id='+id+'</span> <span class="del">x</span> <span class="show">s</span></div>');
				el.data('id', id);
				$('#requests').append( el );
			});
			$('#requests .del').click(function(){
				var req_el = $(this).parent();
				if( confirm('Please confirm to delete request id '+req_el.data('id')) ){				
					$.ajax({
						type: 'DELETE',
						url: rest_base+'/broker/request/'+req_el.data('id'),
						success: function(){
							req_el.remove();
							//$('.req[data-id="'+id+'"]').remove();
						}
					});
				}
			});
			$('#requests .show').click(function(){
				var req_el = $(this).parent();
				window.location.href = 'request.html#'+req_el.data('id');
			});
		},
		dataType: "xml"
	});
}
function localTimeToISO(local_str){
	if( local_str.length == 10 ){
		local_str += "T00:00";
	}
	if( local_str.length == 16 ){
		local_str += ":00";
	}
	
	// TODO parse local_str and convert and return YYYY-MM-DDT00:00:00Z
	local_str += "Z"; // XXX 
	return local_str;
}
function buildRequestDefinitionXml(requestId, fragment){
	var xml = $.parseXML('<queryRequest xmlns="http://aktin.org/ns/exchange" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"><id>'+requestId+'</id><reference/><scheduled/><query><title/><description/><principal><name/><organisation/><email/><phone/></principal><schedule xsi:type="singleExecution"><duration/></schedule></query></queryRequest>');
	$(xml).find('scheduled').text(localTimeToISO($('#new_request input[name="scheduled"]').val()));
	$(xml).find('reference').text(localTimeToISO($('#new_request input[name="reference"]').val()));
	var query = $(xml).find('query')[0];
	// use form elements
	$('#new_request *').filter(':input').each(function(){
		var jpath = $(this).data('jpath');
		if( jpath == '' )return;
		$(xml).find(jpath).text($(this).val());
	});
	// append XML fragment to query
	$(query).append($(fragment).find('*').eq(0));
	return xml;
}

function allocateRequestId(success, error){
			$.post({
				contentType: '',
				url: rest_base+'/broker/request',
				success: function(data, status, xhr) {
					var loc = xhr.getResponseHeader('Location');
					var id = loc.substr(loc.lastIndexOf('/')+1);
					console.log('Request added: '+loc);
					setRequestDefinition(loc, id, success, function(message){
						// all failures from this point on should
						// delete the request
						console.log('Deleting request '+id+'...');
						$.ajax({ 
							type: 'DELETE', 
							contentType: 'application/xml',
							url: loc
						});
						error(message);
					});
				},
				//processData: false,
				//dataType: "xml",
				error: function(x, m, t){
					error('Unable to allocate new request ID: '+m);
				}
			});
}
function buildNodesXml(nodeIds){
	var xml = $.parseXML('<nodes xmlns="http://aktin.org/ns/exchange"></nodes>');
	for( var i=0; i<nodeIds.length; i++ ){
		var el = xml.createElementNS('http://aktin.org/ns/exchange','node');
		el.innerHTML = nodeIds[i];
		xml.documentElement.appendChild(el);
	}
	return xml;
}

function setRequestDefinition(location, id, success_fn, error_fn){
	// load XML syntax fragment
	// content was already verified to be valid XML in addNewRequest()
	var fragment = $.parseXML($('#new_request textarea[name="xml"]').val());
	var data = buildRequestDefinitionXml(id, fragment);
	$.ajax({ 
		type: 'PUT', 
		data: data,
		processData: false,
		contentType: 'application/vnd.aktin.query.request+xml', // use aktin mime type
		url: location,
		success: function() {
			// definition submitted
			if( $("#limit_target_s").prop("checked") ){
				// limit request to certain nodes
				// build <nodes><node>1</node>... and post to location+'/nodes
				var n = $('#target').val();
				var x = buildNodesXml(n);
				console.log('Limiting request target nodes to '+n);
				$.ajax({
					type: 'PUT', 
					data: x,
					processData: false,
					contentType: 'application/xml',
					url: location+'/nodes',
					success: function(){
						console.log('..ok');
						success_fn(location,id);
					},
					error: function(x,m,t){
						console.log('..failed');
						error_fn('Failed to limit destination nodes: '+m);
					}
				});
			}else{
				console.log('Request not limited to certain target nodes');
				success_fn(location, id);
			}
		},
		error: function(x, m, t){
			error_fn('Unable to set request definition for request '+id+': '+m);
		},
		dataType: "xml"
	});

}
function addNewRequest(){
	// validate request syntax
	try{
		var fragment = $.parseXML($('#new_request textarea[name="xml"]').val());
	}catch( e ){
		try{
			var ta = $('#new_request textarea[name="xml"]')[0];
			ta.setCustomValidity("No valid XML!")
			ta.reportValidity();
		}catch( e ){
			// HTML5 form validation reporting failed, display alert box
			alert('query definition not valid XML');
		}
		return;
	}

	// disable submit button
	$('#new_request button').prop('disabled',true);
	allocateRequestId(function(location, id){
		// success
		$('#new_request button').prop('disabled',false);
		// publish immediately
		console.log('Publishing request '+location);
		$.post(location+'/publish');	
		// reload request list
		loadRequestList();
		alert('Request created with id '+id);
		// TODO clear input
		// TODO display status notification of success
	}, 
	function(){
		// failed
		$('#new_request button').prop('disabled',false);
		alert('Unable to create new request id');
	});
}

==> broker-admin-1.1.1-sources.jar: webapp/requests.js <==

function init(){
	$('#new_request').on('submit', function(e){
		e.preventDefault();
		addNewRequest();
	});
	$('#submit_request').on('submit', function(e){
		e.preventDefault();
		addNewRequest();
	});
	// load form script first, then the form html, initialize form afterwards
	loadTemplateTypes();
	loadRequestList();
	// load nodes
	getNodes(function(nodes){
		for( var i=0; i<nodes.length; i++ ){
			$('#target').append($("<option/>",{
	        	value: nodes[i].id,
	        	text: nodes[i].cn,
	        	title: nodes[i].dn
	        }));
		}
		console.log('Loaded nodes: '+nodes.length);
	},function(){
		alert('DWH-Knoten konnten nicht geladen werden.');
	});
	$('#limit_target_a').click(function(){
		// clear selection
		$("#target option:selected").prop("selected", false);
		$("#target").prop("disabled", true);
	});
	$('#limit_target_a').prop("checked", true).trigger("click");
	$('#limit_target_s').click(function(){
		$("#target").prop("disabled", false);
	});
	// hide new request form
	$('#choose_template button').click(function(e){
		e.preventDefault();
		var tt = $('#template_type').val();
		$(this).parent().remove();
		$.getScript('template/'+tt+'/script.js', function(){
			$('#new_request').load('template/'+tt+'/form.html', function(){
				initializeForm();
				$('#create').show();
			});
		});

	});
	$('#create').hide();
	
}

//$(document).ready(function(){
//	init();
//});
Date.prototype.toDateInputValue = (function(){
	var local = new Date(this);
	local.setMinutes(this.getMinutes() - this.getTimezoneOffset());
	return local.toJSON().slice(0,10);
});

function loadTemplateTypes(){
	$.getJSON(rest_base+'/template', function(data){
		console.log("Template",data);
		for( key in data ){
				$('#template_type').append($('<option>', {
    				value: key,
    				text: data[key].title
				}));		
		}
	});
}

function deleteRequest(id){
	if( confirm('Please confirm to delete request id '+id) ){				
		$.ajax({
			type: 'POST',
			url: rest_base+'/broker/request/'+id+'/close',
			success: function(){
				$.ajax({
					type: 'DELETE',
					url: rest_base+'/broker/request/'+id,
					success: function(){
						$('.req[data-id="'+id+'"]').remove();
					}
				});
			}
		});
	}
}

function exportRequest(id){
	var url = rest_base+'/broker/export/request-bundle/'+id;
	resolveIndirectDownload(url, function(new_url){
		// change href
		$('.req[data-id="'+id+'"] .export a').attr('href', new_url);
		// forward to download
		window.location.href = new_url;
	});
}

function loadRequestList(){
	// show info loading
	$('#requests').empty();
	$('#requests').append('<p>Loading ...</p>');
	// load resources
	$.when(
		$.get({url: 'xslt/request-list.xsl', dataType: "xml"}),
		$.get({url: rest_base+'/broker/request', dataType: "xml"})
	).done(function(xsl, xml){
			var xp = new XSLTProcessor();
			xp.importStylesheet(xsl[0]);
			var frag = xp.transformToFragment(xml[0], document);
			$('#requests').empty();
			document.getElementById("requests").appendChild(frag);
	});
}
function localTimeToISO(local_str){
	if( local_str.length == 10 ){
		local_str += "T00:00";
	}
	if( local_str.length == 16 ){
		local_str += ":00";
	}
	
	// TODO parse local_str and convert and return YYYY-MM-DDT00:00:00Z
	local_str += "Z"; // XXX 
	return local_str;
}


function allocateRequestId(success, error){
			$.post({
				contentType: '',
				url: rest_base+'/broker/request',
				success: function(data, status, xhr) {
					var loc = xhr.getResponseHeader('Location');
					var id = loc.substr(loc.lastIndexOf('/')+1);
					console.log('Request added: '+loc);
					setRequestDefinition(loc, id, success, function(message){
						// all failures from this point on should
						// delete the request
						console.log('Deleting request '+id+'...');
						$.ajax({ 
							type: 'DELETE', 
							contentType: 'application/xml',
							url: loc
						});
						error(message);
					});
				},
				//processData: false,
				//dataType: "xml",
				error: function(x, m, t){
					error('Unable to allocate new request ID: '+m);
				}
			});
}
function buildNodesXml(nodeIds){
	var xml = $.parseXML('<nodes xmlns="http://aktin.org/ns/exchange"></nodes>');
	for( var i=0; i<nodeIds.length; i++ ){
		var el = xml.createElementNS('http://aktin.org/ns/exchange','node');
		el.innerHTML = nodeIds[i];
		xml.documentElement.appendChild(el);
	}
	return xml;
}

function setRequestDefinition(location, id, success_fn, error_fn){
	// load XML syntax fragment
	// content was already verified to be valid XML in addNewRequest()
	var data = compileForm(id);
	$.ajax({ 
		type: 'PUT', 
		data: data,
		processData: false,
		contentType: getFormMediaType(), // use aktin mime type
		url: location,
		success: function() {
			// definition submitted
			if( $("#limit_target_s").prop("checked") ){
				// limit request to certain nodes
				// build <nodes><node>1</node>... and post to location+'/nodes
				var n = $('#target').val();
				var x = buildNodesXml(n);
				console.log('Limiting request target nodes to '+n);
				$.ajax({
					type: 'PUT', 
					data: x,
					processData: false,
					contentType: 'application/xml',
					url: location+'/nodes',
					success: function(){
						console.log('..ok');
						success_fn(location,id);
					},
					error: function(x,m,t){
						console.log('..failed');
						error_fn('Failed to limit destination nodes: '+m);
					}
				});
			}else{
				console.log('Request not limited to certain target nodes');
				success_fn(location, id);
			}
		},
		error: function(x, m, t){
			error_fn('Unable to set request definition for request '+id+': '+m);
		},
		dataType: "xml"
	});

}

function addNewRequest(){

	if( validateForm() == false ){
		return;
	}
	// disable submit button
	// TODO XXX make sure the form can not be submitted by pressing enter in a field
	
	$('#submit_request button').prop('disabled',true);
	allocateRequestId(function(location, id){
		// success
		$('#submit_request button').prop('disabled',false);
		// publish immediately
		console.log('Publishing request '+location);
		$.post(location+'/publish');	
		// reload request list
		loadRequestList();
		alert('Request created with id '+id);
		// TODO clear input
		// TODO display status notification of success
	}, 
	function(){
		// failed
		$('#new_request button').prop('disabled',false);
		alert('Unable to create new request id');
	});
}
